"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findServersOnNetwork = exports.findServers = void 0;
/**
 * @module node-opcua-client
 */
const async = require("async");
const client_base_impl_1 = require("../private/client_base_impl");
function findServers(discoveryServerEndpointUri, callback) {
    const client = new client_base_impl_1.ClientBaseImpl({});
    let servers = [];
    let endpoints = [];
    async.series([
        (innerCallback) => {
            client.connect(discoveryServerEndpointUri, innerCallback);
        },
        (innerCallback) => {
            client.findServers((err, _servers) => {
                if (_servers) {
                    servers = _servers;
                }
                innerCallback(err ? err : undefined);
            });
        },
        (innerCallback) => {
            client.getEndpoints({ endpointUrl: undefined }, (err, _endpoints) => {
                if (_endpoints) {
                    endpoints = _endpoints;
                }
                innerCallback(err ? err : undefined);
            });
        },
    ], (err) => {
        client.disconnect(() => {
            callback(err ? err : null, { servers, endpoints });
        });
    });
}
exports.findServers = findServers;
function findServersOnNetwork(discoveryServerEndpointUri, callback) {
    const client = new client_base_impl_1.ClientBaseImpl({});
    client.connect(discoveryServerEndpointUri, (err) => {
        if (!err) {
            client.findServersOnNetwork((err1, servers) => {
                client.disconnect(() => {
                    callback(err1, servers);
                });
            });
        }
        else {
            client.disconnect(() => {
                callback(err);
            });
        }
    });
}
exports.findServersOnNetwork = findServersOnNetwork;
// tslint:disable:no-var-requires
const thenify = require("thenify");
module.exports.findServersOnNetwork =
    thenify.withCallback(module.exports.findServersOnNetwork);
module.exports.findServers =
    thenify.withCallback(module.exports.findServers);
//# sourceMappingURL=findservers.js.map